@@ -18,6 +18,7 @@ from openpyxl.chart import (
     BarChart,
     Reference,
     Series,
+    LineChart,
 )
 from openpyxl.chart.series import DataPoint
 
@@ -31,18 +32,18 @@ from analytics_sdk.utilities import (
     )
 
 #app_name = os.getenv("PLATFORM_ROUTE")
+chart_colors = ["0077C8", "e35420", "32a3df", "673ab7", "9c27b0", "F5BE0A","e30255","a002e3","0079e3","05e3a8","f5d255","f36686","c661e9","24a5ed","62ecbf","F7E487","fb9fb5","e39df0","79ccef","91F4D5","f8ba7e","f3a5ea","cd91eb","87E4F0","cdf38d","f78f4a","eb6dd6","9352e5","5ad1e9","afeb5a","f55a22","e302be","6834E3","02bee3","8ae305"]
 
 class ExcelRenderer:
     ROW_START = 3
     global res
     res = ''
-    def __init__(self, analysis_run, tenantId, integrationId, excel_data, report_gen_start_time, report_gen_completed_time):
+    def __init__(self, analysis_run, tenantId, excel_data, report_gen_start_time, report_gen_completed_time):
         """
         :param analysis_run: AnalysisRun
         """
         self.wb = Workbook()
         self.analysis_run = analysis_run
-        self.integrationId = integrationId
         self.tenantId = tenantId
         self.excel_data = excel_data
         self.report_gen_start_time = report_gen_start_time
@@ -92,7 +93,7 @@ class ExcelRenderer:
             difference = int(et - st)
             return difference
         
-        url = BASE_API_URL + f'/analytics/api/v7/tenants/{tenantId}/{integrationId}/runs/{analysis_run}/summary'
+        url = BASE_API_URL + f'/analytics/api/v3/tenants/{tenantId}/runs/{analysis_run}/summary'
         api_proce_before_time = time.time()
         res = call_requests('GET', url)
         if res == None or not res.ok:
@@ -139,7 +140,8 @@ class ExcelRenderer:
         col_span = len(table_data['data'][0])
 
         ref = f'{get_column_letter(col_start)}{row_start}:{get_column_letter(col_start+col_span-1)}{row_start+row_span-1}'
-        table_name = f'Table{random.randint(0, 100)}'
+        ts = time.time()
+        table_name = f'Table{random.randint(0, 100)}_{ts}'
 
         table = Table(displayName=table_name, ref=ref, tableStyleInfo=TableStyleInfo(name="TableStyleMedium9"))
         ws.add_table(table)
@@ -201,6 +203,55 @@ class ExcelRenderer:
         s1.graphicalProperties.line.solidFill = "0077C8"
         s1.graphicalProperties.solidFill = "0077C8"
 
+        for idx, s in enumerate(chart.series):
+            index = len(chart_colors)%(idx+1)
+            if index >= len(chart_colors):
+                index = 0
+            s.graphicalProperties.line.solidFill = chart_colors[index]
+            s.graphicalProperties.solidFill = chart_colors[index]
+
+        ws.add_chart(chart, chart_data['chart-position'])
+
+    def add_stack_bar_chart(self, ws, chart_data):
+        """
+        add stack bar chart component
+        """
+        row_start = ExcelRenderer.ROW_START + chart_data['start-row']
+        col_start = chart_data['start-col']
+        self._load_data(ws, chart_data['data'], row_start, col_start, False)
+        row_span = len(chart_data['data'])
+        no_of_labels = chart_data['no-of-labels']
+        if not no_of_labels:
+            no_of_labels = 1
+
+        chart = BarChart()
+        chart.type = "col"
+        chart.title = chart_data['chart-title']
+        chart.x_axis.title = chart_data.get('x-axis-title')
+        chart.y_axis.title = chart_data.get('y-axis-title')
+
+        labels = Reference(ws, min_col=col_start, min_row=row_start+1, max_row=row_start+row_span-1)
+        data = Reference(ws, min_col=col_start+1, min_row=row_start, max_row=row_start+row_span-1, max_col=col_start+no_of_labels)
+        chart.add_data(data, titles_from_data=True)
+        chart.set_categories(labels)
+        chart.style = 4
+        if no_of_labels == 1:
+            chart.legend = None
+        chart.grouping = "stacked"
+        chart.width = chart_data.get('width', 20)
+        chart.height = chart_data.get('height', 10)
+
+        s1 = chart.series[0]
+        s1.graphicalProperties.line.solidFill = "0077C8"
+        s1.graphicalProperties.solidFill = "0077C8"
+
+        for idx, s in enumerate(chart.series):
+            index = len(chart_colors)%(idx+1)
+            if index >= len(chart_colors):
+                index = 0
+            s.graphicalProperties.line.solidFill = chart_colors[index]
+            s.graphicalProperties.solidFill = chart_colors[index]
+
         ws.add_chart(chart, chart_data['chart-position'])
 
     def add_scatter_chart(self, ws, chart_data):
@@ -212,6 +263,10 @@ class ExcelRenderer:
         x_axis_date_format = chart_data.get('x-axis-date-format')
         self._load_data(ws, chart_data['data'], row_start, col_start, False, x_axis_date_format)
         row_span = len(chart_data['data'])
+        no_of_lines = chart_data.get('no-of-lines')
+        if not no_of_lines:
+            no_of_lines = 1
+        colors = ["0077C8", "e35420", "32a3df", "673ab7", "9c27b0"]
 
         chart = ScatterChart()
         chart.style = 5
@@ -222,23 +277,65 @@ class ExcelRenderer:
         chart.width = chart_data.get('width', 15)
         chart.height = chart_data.get('height', 7.5)
 
-        labels = Reference(ws, min_col=col_start, min_row=row_start+1, max_row=row_start+row_span-1)
-        data = Reference(ws, min_col=col_start+1, min_row=row_start, max_row=row_start+row_span-1)
-        series = Series(data, labels, title_from_data=True)
-        chart.series.append(series)
-
-        s1 = chart.series[0]
-        s1.marker.symbol = "circle"
-        s1.marker.graphicalProperties.solidFill = "0077C8"  # Marker filling
-        s1.marker.graphicalProperties.line.solidFill = "0077C8"  # Marker outline
-        s1.graphicalProperties.line.solidFill = "0077C8"
-        s1.graphicalProperties.line.width = 24050  # width in EMUs
-        if x_axis_date_format:
-            chart.x_axis.number_format = 'm/d'
-            chart.x_axis.majorTimeUnit = "days"
+        for i in range(no_of_lines):
+            labels = Reference(ws, min_col=col_start, min_row=row_start+i+1, max_row=row_start+row_span-1)
+            data = Reference(ws, min_col=col_start+i+1, min_row=row_start, max_row=row_start+row_span-1)
+            series = Series(data, labels, title_from_data=True)
+            series.marker.symbol = "circle"
+            series.marker.graphicalProperties.solidFill = colors[i % 5]  # Marker filling
+            series.marker.graphicalProperties.line.solidFill = colors[i % 5]  # Marker outline
+            series.graphicalProperties.line.solidFill = colors[i % 5]
+            series.graphicalProperties.line.width = 24050  # width in EMUs
+            if x_axis_date_format:
+                chart.x_axis.number_format = 'm/d'
+                chart.x_axis.majorTimeUnit = "days"
+            chart.series.append(series)
 
         ws.add_chart(chart, chart_data['chart-position'])
 
+    def add_line_chart(self, ws, chart_data):
+        """
+        add line chart component
+        """
+        row_start = ExcelRenderer.ROW_START + chart_data['start-row']
+        col_start = chart_data['start-col']
+        x_axis_date_format = chart_data.get('x-axis-date-format')
+        self._load_data(ws, chart_data['data'], row_start, col_start, False, x_axis_date_format)
+        row_span = len(chart_data['data'])
+        no_of_lines = chart_data.get('no-of-lines')
+        if not no_of_lines:
+            no_of_lines = 1
+        colors = ["0077C8", "e35420", "32a3df", "673ab7", "9c27b0"]
+
+        chart = LineChart()
+        chart.style = 5
+        chart.legend.position = 'b'     # Location of legend
+        if no_of_lines == 1:
+            chart.legend = None
+        chart.title = chart_data['chart-title']
+        chart.x_axis.title = chart_data.get('x-axis-title')
+        chart.y_axis.title = chart_data.get('y-axis-title')
+        chart.width = chart_data.get('width', 20)
+        chart.height = chart_data.get('height', 10)
+
+        for i in range(no_of_lines):
+            labels = Reference(ws, min_col=col_start, min_row=row_start+i+1, max_row=row_start+row_span-1)
+            data = Reference(ws, min_col=col_start+i+1, min_row=row_start, max_row=row_start+row_span-1)
+            chart.add_data(data, titles_from_data=True)
+            chart.set_categories(labels)
+            series = chart.series[i]
+            series.marker.graphicalProperties.solidFill = colors[i % 5]  # Marker filling
+            series.marker.graphicalProperties.line.solidFill = colors[i % 5]  # Marker outline
+            series.graphicalProperties.line.solidFill = colors[i % 5]
+            series.graphicalProperties.line.width = 24050  # width in EMUs
+            series.smooth=True        # Draw a smooth line
+            if x_axis_date_format:
+                chart.x_axis.number_format = 'm/d'
+                chart.x_axis.majorTimeUnit = "days"
+
+        ws.add_chart(chart, chart_data['chart-position'])
+
+
     def generate_summary(self, res, tenantId, report_gen_start_time, report_gen_completed_time):
         """
         add summary sheet
@@ -386,6 +483,10 @@ class ExcelRenderer:
             self.add_bar_chart(ws, component_data)
         elif component_data['type'] == 'scatter-chart':
             self.add_scatter_chart(ws, component_data)
+        elif component_data['type'] == 'line-chart':
+            self.add_line_chart(ws, component_data)
+        elif component_data['type'] == 'stack-bar-chart':
+            self.add_stack_bar_chart(ws, component_data)
 
     def render(self):
         """
