@@ -35,7 +35,7 @@ API_TIME_STACKS = int(os.getenv('API_TIME_STACKS', 5))
 from .renderer.excel import ExcelRenderer
 storage_name = os.getenv('STORAGE_NAME')
 
-global ACCESS_KEY_ID, SECRET_ACCESS_KEY, REGION_NAME, BUCKET_NAME
+global ACCESS_KEY_ID, SECRET_ACCESS_KEY, REGION_NAME, BUCKET_NAME, ACCESS_HEADER, TOKEN_RETRIES
 
 if storage_name == 's3':
     ACCESS_KEY_ID = os.getenv('ACCESS_KEY_ID')
@@ -49,7 +49,8 @@ elif storage_name == 'ceph':
     ENDPOINT_URL = os.getenv('ENDPOINT_URL')
 else:
     logger.info("No storages are found")
-
+TOKEN_RETRIES = 0
+ACCESS_HEADER = None
 
 def get_token():
     headers = {"Content-Type" : "application/x-www-form-urlencoded" , "Accept" : "application/json"};
@@ -90,13 +91,38 @@ def login_get_headers():
     return headers
 
 
+def get_auth_token():
+    logging.info('invoking auth token call...')
+    headers = {"Content-Type" : "application/x-www-form-urlencoded" , "Accept" : "application/json"}
+    post_data = {"grant_type": "client_credentials", "client_id" : API_KEY, "client_secret" : API_SECRET}
+    token_url = BASE_API_URL + "/auth/oauth/token"
+    response = requests.post(token_url,data=post_data,headers=headers,verify=False)
+    json = response.json()
+    auth = str(json["token_type"]) + " " + str(json["access_token"])
+    global ACCESS_HEADER
+    ACCESS_HEADER = {"Content-Type" : "application/json" ,"Accept" : "application/json" , "Authorization" : auth }
+
+
 def call_requests(method, url, params=None, data=None, json=None, verify=True):
-    headers = get_headers()
     retry = 1
     resp = None
+    if ACCESS_HEADER is None:
+        get_auth_token()
+        global TOKEN_RETRIES
+        TOKEN_RETRIES = 0
     while retry <= API_RETRY:
         try:
-            resp = requests.request(method, url, params=params, data=data, json=json, headers=headers, verify=verify)
+            resp = requests.request(method, url, params=params, data=data, json=json, headers=ACCESS_HEADER, verify=verify)
+            if resp.status_code == 407:
+                if TOKEN_RETRIES < 5:
+                    logging.info(f'Token expired, re-generating token..{TOKEN_RETRIES}' )
+                    TOKEN_RETRIES +=1
+                    time.sleep(TOKEN_RETRIES * 2)
+                    get_auth_token()
+                    call_requests(method, url, params=params, data=data, json=json, verify=verify)
+                else:
+                    TOKEN_RETRIES = 0
+                    raise Exception(f'API Fetching failed {url}')
             if not resp.ok:
                 time.sleep(retry * 2)
                 retry+=1
@@ -105,7 +131,8 @@ def call_requests(method, url, params=None, data=None, json=None, verify=True):
             time.sleep(retry * 2)
             retry+=1
             continue
-
+        
+        TOKEN_RETRIES = 0
         return resp
     
     return resp
@@ -128,7 +155,7 @@ def login_call_requests(method, url, params=None, data=None, json=None, verify=T
             continue
 
         return resp
-    
+
     return resp
 
 
@@ -335,13 +362,38 @@ def generate_pdf(analysis_run):
     }
 
     api_proce_before_time = time.time()
-    res = requests.post(url, data=data).json()
-    api_proce_after_time = time.time()
-    api_proce_diff = diff_sec(api_proce_before_time, api_proce_after_time)
-    if api_proce_diff > API_TIME_STACKS:
-        logging.info('Pdf response is took %d (greater than %d) seconds', api_proce_diff, API_TIME_STACKS)
-    logger.info('Pdf response is: %s', res['Key'])
-    return res['Key']
+    res = {}
+    retry = 1
+    while retry <= 3:
+        try:
+            resp = requests.post(url, data=data)
+            if not resp.ok:
+                logging.info(f'{analysis_run} pdf generation failed.. retrying.. {retry} ')
+                time.sleep(retry * 2)
+                retry+=1
+                continue
+            if resp and resp is not None:
+                res = resp.json()
+                api_proce_after_time = time.time()
+                api_proce_diff = diff_sec(api_proce_before_time, api_proce_after_time)
+                if api_proce_diff > API_TIME_STACKS:
+                    logging.info('%s , pdf response is took %d (greater than %d) seconds', analysis_run, api_proce_diff, API_TIME_STACKS)
+                if 'Key' in res:
+                    logger.info('pdf response is: %s', res['Key'])
+                    return res['Key']
+                else:
+                    logging.info(f'key not found, pdf generation failed, returning empty value for runid ::: {analysis_run}')
+                    raise Exception(f'key not found, pdf generation failed for run ::: {analysis_run}')
+        except requests.exceptions.ConnectionError:
+            time.sleep(retry * 2)
+            retry+=1
+            continue
+        except Exception as e:
+            raise Exception(f'pdf generation failed for run ::: {analysis_run}, Exception is : {e}')
+
+    logging.info(f'pdf generationg failed after max tries, returning empty value for runid ::: {analysis_run}')
+    raise Exception(f'pdf generation failed after max tries, for run ::: {analysis_run}')
+        
 
 
 
@@ -479,12 +531,11 @@ def generate_excel_file(run_id, orgId, report_gen_start_time, report_gen_complet
 def init_mail(org_id, run_id):
     logger.info('Entered into init_mail method')
     try:
-        url = BASE_API_URL + f'/analytics/api/v3/tenants/{org_id}/sends'
-        data={
-                "runId" : run_id
-            }
+        # /tenants/{tenantId}/runs/{id}/sends
+        url = BASE_API_URL + f'/analytics/api/v3/tenants/{org_id}/runs/{run_id}/sends'
+        data = {}
         t1 = int(time.time())
-        res = call_post_requests(url , data=json.dumps(data), verify=False);
+        res = call_post_requests(url , data=json.dumps(data), verify=False)
         t2 = int(time.time())
         duration = t2-t1
         if duration > API_TIME_STACKS:
@@ -567,3 +618,84 @@ def get_last_run(summary_res):
 
 def get_app_description(summary_res):
     return summary_res['app_description']
+
+
+def get_paginated_api_results(method, url, data, type, add_pageNo=True):
+    resp = []
+    page_no=1
+    nextPage= True
+    pageSize= 100
+
+    try:
+        while (nextPage != False):
+            data = json.loads(data)
+            if "pageNo" in data:
+                data["pageNo"] = page_no
+                data["pageSize"] = pageSize
+            data=json.dumps(data)
+            if method == 'POST':
+                res = get_post_request_results(url, data, type)
+            else:
+                if add_pageNo:
+                    uri = url + f'&pageNo={page_no}&pageSize={pageSize}'
+                else:
+                    uri = url
+                res = get_response(uri, type)
+
+            if res == None or not res.ok:
+                logger.error('Get %s API is failed, url %s', type, url)
+                return resp
+            elif "results" not in res.json() or len(res.json()['results'])==0 :
+                logger.error('Get %s API results are empty, url is %s', type, url)
+                return None
+            else:   
+                result = res.json()['results']
+                resp.append(result)
+        
+                if "nextPage" not in res.json():
+                    resp = []
+                    logger.error('Get %s nextPage keyword is missing in url %s', type, url)
+                    resp = [item for sublist in resp for item in sublist]
+                    return resp
+        
+                nextPage=res.json()['nextPage']
+                page_no+=1
+        
+        resp = [item for sublist in resp for item in sublist] # To eliminate list of list (ex: [[data:{}]] -> [data:{}])
+    except Exception as e:
+        traceback.print_exc()
+    return resp
+
+
+def get_post_request_results(url, data, type):
+    start_time = int(time.time())
+    logging.info(f'api type: {type}, : url : {url}')
+
+    res = call_post_requests(url, data=data, verify=False)
+    duration = int(time.time()) - start_time
+    if duration > API_TIME_STACKS:
+        logging.info(f'Get {type} API response took %d (greater than %d) seconds, url : {url}', duration, API_TIME_STACKS)
+    return res
+
+
+def get_post_opsql_count_results(url, data, type):
+    res = get_post_request_results(url, data, type)
+    if res == None or not res.ok:
+        logger.error('FAILED :' + type)
+        return None
+    elif "count" not in res.json() or len(res.json()) == 0:
+        logger.error('Result are Empty for ' + type)
+        return None
+    else:
+        resp = res.json()
+
+    return resp
+
+
+def update_run_progress(tenant_id, run_id, percent, message):
+    url = BASE_API_URL + f'/analytics/api/v3/tenants/{tenant_id}/runs/{run_id}'
+    data = {
+        "percent" : percent,
+        "message" : message
+    }
+    #TODO : Implement logic here to update the run
\ No newline at end of file
